import cv2 as cv
import numpy as np

"""Author: Ben Miller
"""


def display(cap):
    global hsv_img
    lower = np.array([0, 0, 103])
    upper = np.array([177, 107, 204])
    morph_shape = cv.MORPH_RECT  # max is 2 0-2
    dilation_size = 2
    erozion_size = 5  # max is 21 0-21
    threshold = 150

    element_erode = cv.getStructuringElement(
        morph_shape,
        (2 * erozion_size + 1, 2 * erozion_size + 1),
        (erozion_size, erozion_size),
    )
    element_dilate = cv.getStructuringElement(
        morph_shape,
        (2 * dilation_size + 1, 2 * dilation_size + 1),
        (dilation_size, dilation_size),
    )

    _, img = cap.read()
    drawing = np.zeros((img.shape[0], img.shape[1], 3), dtype=np.uint8)

    while True:
        _, img = cap.read()

        lower[0] = cv.getTrackbarPos("H-Low", "Live Video")
        upper[0] = cv.getTrackbarPos("H-High", "Live Video")
        lower[1] = cv.getTrackbarPos("S-Low", "Live Video")
        upper[1] = cv.getTrackbarPos("S-High", "Live Video")
        lower[2] = cv.getTrackbarPos("V-Low", "Live Video")
        upper[2] = cv.getTrackbarPos("V-High", "Live Video")

        hsv_img = cv.cvtColor(img, cv.COLOR_RGB2HSV)

        mask = cv.inRange(hsv_img, lower, upper)
        mask = cv.dilate(mask, element_dilate)
        mask = cv.erode(mask, element_erode)

        contours, _ = cv.findContours(mask, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
        if len(contours) != 0:
            # Find the biggest countour by area
            c_area_list = [cv.contourArea(x) for x in contours]
            c_index = c_area_list.index(max(c_area_list))
            x, y, w, h = cv.boundingRect(contours[c_index])
            # Draw the biggest contour in green on live video
            cv.rectangle(img, (x, y), (x + w, y + h), (0, 255, 0), 2)
            # Clear drawing
            drawing.fill(0)
            # Draws the largest contour on the contour screen
            cv.drawContours(drawing, contours, c_index, (255, 255, 255), 3)

        cv.imshow("Live Video", img)
        cv.imshow("HSV Video", hsv_img)
        cv.imshow("Ball", mask)
        cv.imshow("Largest Contour", drawing)

        k = cv.waitKey(1)
        if k == 113 or k == 27:  # q key and esc
            break


def process_click(event, x, y, _, __):
    if event == cv.EVENT_LBUTTONDOWN:  # checks mouse left button down condition
        print("HSV: ", hsv_img[y, x])


def nothing(_):
    ...


def main():
    print("start")
    cap = cv.VideoCapture(0)
    cv.namedWindow("Live Video")
    cv.createTrackbar("H-Low", "Live Video", 0, 179, nothing)
    cv.createTrackbar("H-High", "Live Video", 0, 179, nothing)
    cv.createTrackbar("S-Low", "Live Video", 0, 255, nothing)
    cv.createTrackbar("S-High", "Live Video", 0, 255, nothing)
    cv.createTrackbar("V-Low", "Live Video", 0, 255, nothing)
    cv.createTrackbar("V-High", "Live Video", 0, 255, nothing)

    cv.namedWindow("HSV Video")
    cv.setMouseCallback("HSV Video", process_click)

    cv.namedWindow("Ball")

    cv.namedWindow("Largest Contour")

    display(cap)

    cv.destroyAllWindows()
    cap.release()
    print("end")


if __name__ == "__main__":
    main()