import cv2 as cv
import numpy as np

"""Author: Ben Miller
"""


def display(cap):
    contour_threshold = 2000
    bw_threshold = 20
    bw_high_threshold = 230
    brightness = 35
    update_weight = 0.4

    _, frame = cap.read()
    width, height, _ = frame.shape
    image_ave = np.zeros((width, height, 3), np.float32)
    bright = brightness * np.ones((width, height, 3), np.uint8)
    contours_img = np.zeros((width, height, 1), dtype=np.uint8)

    while True:
        _, frame = cap.read()

        bright_frame = np.add(frame, bright)
        blur = cv.GaussianBlur(bright_frame, (5, 5), 0)
        running_color_depth = cv.absdiff(
            bright_frame,
            cv.convertScaleAbs(
                cv.accumulateWeighted(
                    blur,
                    image_ave,
                    update_weight,
                )
            ),
        )

        bw = cv.cvtColor(running_color_depth, cv.COLOR_RGB2GRAY)
        _, bw = cv.threshold(bw, bw_threshold, 255, cv.THRESH_BINARY + cv.THRESH_OTSU)
        bw = cv.GaussianBlur(bw, (5, 5), 0)
        _, bw = cv.threshold(
            bw, bw_high_threshold, 255, cv.THRESH_BINARY + cv.THRESH_OTSU
        )

        # Clear drawing
        contours_img.fill(255)
        contours, _ = cv.findContours(bw, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
        for index, contour in enumerate(contours):
            if cv.contourArea(contour) >= contour_threshold:
                x, y, w, h = cv.boundingRect(contour)
                # Draw the biggest contour in green on live video
                cv.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)
                # Draws the largest contour on the contour screen
                cv.drawContours(contours_img, contours, index, (0), 3)

        cv.imshow("Live Video", frame)
        cv.imshow("Temporal", running_color_depth)
        cv.imshow("BW", bw)
        cv.imshow("Contours", contours_img)

        k = cv.waitKey(1)
        if k == 113 or k == 27:  # q key and esc
            break


def main():
    print("start")
    cap = cv.VideoCapture(0)

    cv.namedWindow("Live Video")
    cv.namedWindow("BW")
    cv.namedWindow("Temporal")
    cv.namedWindow("Contours")

    display(cap)

    cv.destroyAllWindows()
    cap.release()
    print("end")


if __name__ == "__main__":
    main()